# vim: set fileencoding=utf-8 :
# TODO: Replace all aggregate query to DB API when it'll be work in Django.
# See: http://code.djangoproject.com/ticket/3566
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.conf import settings
from katrin.models import Users
from traff.models import ServiceTraffFilters
import datetime, time
import socket, struct
import operator
from calendar import *
from dateutil.relativedelta import relativedelta
from django.db import connection
from django.contrib.auth.decorators import login_required

class days_of_month:
    def __init__(self, number, download, upload, download_cost, upload_cost):
	self.number = number
	self.download = download
	self.upload = upload
	self.download_cost = download_cost
	self.upload_cost = upload_cost
	self.cost = download_cost + upload_cost

class hours_of_day(days_of_month):
	pass

class months_of_year(days_of_month):
	pass


class Traffic:
    def __init__(self, download, upload, cost_download, cost_upload):
		self.download = download
		self.upload = upload
		self.cost_download = cost_download
		self.cost_upload = cost_upload

class UTraffic(Traffic):
    def __init__(self, user, download, upload, cost_download, cost_upload):
		Traffic.__init__(self, download, upload, cost_download, cost_upload)
		self.user = user

class traff_by_ports_and_protocols:
    def __init__(self, direction, proto, src_port, dst_port, bytes, cost):
	    self.direction = direction
	    self.proto = proto
	    self.src_port = src_port
	    self.dst_port = dst_port
	    self.bytes = bytes
	    self.cost = cost

class traff_by_ip(Traffic):
    def __init__(self, ip, download, upload, cost_download, cost_upload):
		Traffic.__init__(self, download, upload, cost_download, cost_upload)
		self.ip = ip

class traff_by_filter(Traffic):
    def __init__(self, filter, download, upload, cost_download, cost_upload):
		Traffic.__init__(self, download, upload, cost_download, cost_upload)
		self.filter = filter

@login_required()


def show_menu(request):
        object_list = Users.objects.order_by('id')
        return render_to_response('traff/menu.html',
                        {'users': object_list,
						'year': datetime.date.today().year,
						'month': datetime.date.today().month,
						'day': datetime.date.today().day,
						'hour': datetime.datetime.today().hour,
						'WITHOUT_COST': settings.WITHOUT_COST
						},
			context_instance=RequestContext(request))

def show_user_stats_year(request, user_id, year):
	year = int(year)

	date_today = datetime.date.today()
	if (year==0):
		year = date_today.year
	user = Users.objects.get(id=user_id)

	year_download = 0
	year_upload = 0
	year_download_cost = 0
	year_upload_cost = 0

	moy=[]
	for month in range(1,13):
		start_date = datetime.datetime(year, month, 1, 0, 0, 0)
		row = get_aggregate_stats(user_id, start_date, 'month')[0]

		month_download = bytes2Xbytes(0)
		month_upload = bytes2Xbytes(0)
		month_download_cost = 0
		month_upload_cost = 0
		if row!=None:
			if row[0]!=None:
				year_download += row[0]
				month_download = bytes2Xbytes(row[0])
			if row[2]!=None:
				year_upload += row[2]
				month_upload = bytes2Xbytes(row[2])
			if row[1]!=None:
				month_download_cost = row[1]
				year_download_cost += row[1]
			if row[3]!=None:
				month_upload_cost = row[3]
				year_upload_cost += row[3]

		moy.append(months_of_year(month, month_download, month_upload, month_download_cost, month_upload_cost))

	year_upload = bytes2Xbytes(year_upload)
	year_download = bytes2Xbytes(year_download)

	year_cost = year_download_cost + year_upload_cost

	return render_to_response('traff/user_stats_year.html', {
				'year_download': year_download,
				'year_upload': year_upload,
				'year_download_cost': year_download_cost,
				'year_upload_cost': year_upload_cost,
				'year_cost': year_cost,
				'user': user,
				'year': year,
				'months_of_year': moy,
				'WITHOUT_COST': settings.WITHOUT_COST
				},
			context_instance=RequestContext(request))




def show_user_stats_month(request, user_id, year, month):
	year = int(year)
	month = int(month)

	date_today = datetime.date.today()
	if (year==0):
		year = date_today.year
	if (month==0):
		month = date_today.month
	user = Users.objects.get(id=user_id)

	month_download = 0
	month_upload = 0

	month_download_cost = 0
	month_upload_cost = 0

	dom=[]
	for day in range(1,monthrange(year,month)[1]+1):
		start_date = datetime.datetime(year, month, day, 0, 0, 0)
		row = get_aggregate_stats(user_id, start_date, 'day')[0]

		day_download = bytes2Xbytes(0)
		day_upload = bytes2Xbytes(0)
		day_download_cost = 0
		day_upload_cost = 0
		if row!=None:
			if row[0]!=None:
				month_download = month_download + row[0]
				day_download = bytes2Xbytes(row[0])
			if row[2]!=None:
				month_upload = month_upload + row[2]
				day_upload = bytes2Xbytes(row[2])
			if row[1]!=None:
				month_download_cost += row[1]
				day_download_cost = row[1]
			if row[3]!=None:
				month_upload_cost += row[3]
				day_upload_cost = row[3]

		dom.append(days_of_month(day, day_download, day_upload, day_download_cost, day_upload_cost))

	month_upload = bytes2Xbytes(month_upload)
	month_download = bytes2Xbytes(month_download)

	month_cost = month_download_cost + month_upload_cost

	#traff by filter
	rows = get_aggregate_stats_by_filter(user_id, start_date, "month")
	tbfilter = get_traff_by_filter(rows)

	return render_to_response('traff/user_stats_month.html', {
				'month_download': month_download,
				'month_upload': month_upload,
				'month_download_cost': month_download_cost,
				'month_upload_cost': month_upload_cost,
				'month_cost': month_cost,
				'month': month,
				'user': user,
				'year': year,
				'days_of_month': dom,
				'traff_by_filter': tbfilter,
				'WITHOUT_COST': settings.WITHOUT_COST
				},
			context_instance=RequestContext(request))


def show_user_stats_day(request,user_id,year,month,day):
	year = int(year)
	month = int(month)
	day = int(day)

	date_today = datetime.date.today()
	if (year==0):
		year = date_today.year
	if (month==0):
		month = date_today.month
	if (day==0):
		day = date_today.day

	user = Users.objects.get(id=user_id)

	day_download = 0
	day_upload = 0
	day_download_cost = 0
	day_upload_cost = 0

	hod=[]
	for hour in range(0, 24):
		start_date = datetime.datetime(year, month, day, hour, 0, 0)
		row = get_aggregate_stats(user_id, start_date, 'hour')[0]

		hour_download = bytes2Xbytes(0)
		hour_upload = bytes2Xbytes(0)
		hour_download_cost = 0
		hour_upload_cost = 0
		if row!=None:
			if row[0]!=None:
				day_download = day_download + row[0]
				hour_download = bytes2Xbytes(row[0])
			if row[2]!=None:
				day_upload = day_upload + row[2]
				hour_upload = bytes2Xbytes(row[2])
			if row[1]!=None:
				hour_download_cost = row[1]
				day_download_cost += row[1]
			if row[3]!=None:
				hour_upload_cost = row[3]
				day_upload_cost += row[3]

		hod.append(hours_of_day(hour, hour_download, hour_upload, hour_download_cost, hour_upload_cost))

	day_upload = bytes2Xbytes(day_upload)
	day_download = bytes2Xbytes(day_download)
	
	day_cost = day_upload_cost + day_download_cost
	
	#traff by filter
	rows = get_aggregate_stats_by_filter(user_id, start_date, "day")
	tbfilter = get_traff_by_filter(rows)


	return render_to_response('traff/user_stats_day.html', {
				'day_download': day_download,
				'day_upload': day_upload,
				'day_download_cost': day_download_cost,
				'day_upload_cost': day_upload_cost,
				'day_cost': day_cost,
				'month': month,
				'user': user,
				'year': year,
				'day': day,
				'hours_of_day': hod,
				'traff_by_filter': tbfilter,
				'WITHOUT_COST': settings.WITHOUT_COST
				},
			context_instance=RequestContext(request))


def show_user_stats_hour(request,user_id,year,month,day,hour):
	# amount first ip with max bytes
	amount_ip = settings.AMOUNT_IP

	year = int(year)
	month = int(month)
	day = int(day)
	hour = int(hour)

	start_date = datetime.datetime(year, month, day, hour, 0, 0)

	#traff by ip
	rows = get_stats(user_id, start_date, 'hour', 'addr', amount_ip)
	tbip=[]
	for row in rows:
		bytes_download = bytes2Xbytes(row[1])
		bytes_upload = bytes2Xbytes(row[2])
		cost_download = "%1.2f" % (float(row[3]))
		cost_upload = "%1.2f" % (float(row[4]))
		tbip.append(traff_by_ip(row[0], bytes_download, bytes_upload, cost_download, cost_upload))

	#traff by filter
	rows = get_stats(user_id, start_date, 'hour', 'filterid', 0)
	tbfilter = get_traff_by_filter(rows)

	#by ports
	rows = get_stats(
			user_id=user_id, 
			start_date=start_date,
			interval='hour',
			group_by='in_traff, proto, src_port',
			limit=0,
			fields="in_traff,proto,src_port,dst_port,sum(bytes),sum(cost)")
	pp=[]
	for row in rows:
		bytes = bytes2Xbytes(row[4])
		cost = "%1.2f" % (float(row[5]))
		pp.append(traff_by_ports_and_protocols(row[0], row[1], row[2], row[3], bytes, cost));

	return render_to_response('traff/user_stats_hour.html',
			{
			'amount_ip': amount_ip,
			'traff_by_ip': tbip,
			'traff_by_filter': tbfilter,
			'traff_by_ports_and_protocols': pp,
			'year': year,
			'month': month,
			'day': day,
			'hour': hour,
			'WITHOUT_COST': settings.WITHOUT_COST
			},
			context_instance=RequestContext(request))



def show_users_stats_month(request,year,month):
	date_today = datetime.date.today()

	if (request.POST):
		year = request.POST.get('year', str(date_today.year))
		month = request.POST.get('month', str(date_today.month))
	year = int(year)
	month = int(month)

	start_date = datetime.datetime(year, month, 1, 0, 0, 0)

	traff=[]

	users = Users.objects.order_by('id')

	all_download = 0
	all_upload = 0
	all_cost_download = 0
	all_cost_upload = 0

	for user in users:
		#get download, upload and cost for it
		row = get_aggregate_stats(user.id, start_date, 'month')[0]
		if row[0] is None:
			download = 0
		else:
			download = row[0]
		all_download = all_download + download

		if row[2] is None:
			upload = 0
		else:
			upload = row[2]
		all_upload = all_upload + upload

		if row[1] is None:
			cost_download = 0
		else:
			cost_download = row[1]
		all_cost_download = all_cost_download + cost_download
	
		if row[3] is None:
			cost_upload = 0
		else:
			cost_upload = row[3]
		all_cost_upload = all_cost_upload + cost_upload

		
		traff.append(UTraffic(user, download, upload, cost_download, cost_upload))

	traff.sort(key=operator.attrgetter('upload'),reverse=True)
	traff.sort(key=operator.attrgetter('download'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_upload'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_download'),reverse=True)

	all_download = bytes2Xbytes(all_download)
	all_upload = bytes2Xbytes(all_upload)
	all_cost_download = "%1.2f" % (all_cost_download)
	all_cost_upload = "%1.2f" % (all_cost_upload)

	for tr in traff:
		tr.cost_download = "%1.2f" % (tr.cost_download)
		tr.cost_upload = "%1.2f" % (tr.cost_upload)
		tr.download = bytes2Xbytes(tr.download)
		tr.upload = bytes2Xbytes(tr.upload)

	#traff by filter
	rows = get_aggregate_stats_by_filter(0, start_date, "month")
	tbfilter=get_traff_by_filter(rows)

	return render_to_response('traff/users_stats_month.html',
			{'traff': traff,
			'month': month,
			'months': range(1,13),
			'year': year,
			'years': range(date_today.year-3,date_today.year+1),
			'all_download': all_download,
			'all_upload': all_upload,
			'all_cost_download': all_cost_download,
			'all_cost_upload': all_cost_upload,
			'traff_by_filter': tbfilter,
			'WITHOUT_COST': settings.WITHOUT_COST
			},
			context_instance=RequestContext(request)
			)


def show_users_stats_day(request,year,month,day):
	date_today = datetime.date.today()

	if (request.POST):
		year = request.POST.get('year', str(date_today.year))
		month = request.POST.get('month', str(date_today.month))
		day = request.POST.get('day', str(date_today.day))
	year = int(year)
	month = int(month)
	day = int(day)

	start_date = datetime.datetime(year, month, day, 0, 0, 0)

	traff=[]

	users = Users.objects.order_by('id')

	all_download = 0
	all_upload = 0
	all_cost_download = 0
	all_cost_upload = 0

	for user in users:
		#get download, upload and cost for it
		row = get_aggregate_stats(user.id, start_date, 'day')[0]
		if row[0] is None:
			download = 0
		else:
			download = row[0]
		all_download = all_download + download

		if row[2] is None:
			upload = 0
		else:
			upload = row[2]
		all_upload = all_upload + upload

		if row[1] is None:
			cost_download = 0
		else:
			cost_download = row[1]
		all_cost_download = all_cost_download + cost_download
	
		if row[3] is None:
			cost_upload = 0
		else:
			cost_upload = row[3]
		all_cost_upload = all_cost_upload + cost_upload

		
		traff.append(UTraffic(user, download, upload, cost_download, cost_upload))

	traff.sort(key=operator.attrgetter('upload'),reverse=True)
	traff.sort(key=operator.attrgetter('download'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_upload'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_download'),reverse=True)


	all_download = bytes2Xbytes(all_download)
	all_upload = bytes2Xbytes(all_upload)
	all_cost_download = "%1.2f" % (all_cost_download)
	all_cost_upload = "%1.2f" % (all_cost_upload)

	for tr in traff:
		tr.cost_download = "%1.2f" % (tr.cost_download)
		tr.cost_upload = "%1.2f" % (tr.cost_upload)
		tr.download = bytes2Xbytes(tr.download)
		tr.upload = bytes2Xbytes(tr.upload)

	#traff by filter
	rows = get_aggregate_stats_by_filter(0, start_date, "day")
	tbfilter=get_traff_by_filter(rows)

	return render_to_response('traff/users_stats_day.html',
			{'traff': traff,
			'month': month,
			'months': range(1,13),
			'year': year,
			'years': range(date_today.year-3,date_today.year+1),
			'day': day,
			'days': range(1,monthrange(year,month)[1]+1),
			'all_download': all_download,
			'all_upload': all_upload,
			'all_cost_download': all_cost_download,
			'all_cost_upload': all_cost_upload,
			'traff_by_filter': tbfilter,
			'WITHOUT_COST': settings.WITHOUT_COST
			},
			context_instance=RequestContext(request)
			)

def show_users_stats_hour(request,year,month,day,hour):
	date_today = datetime.datetime.today()

	if (request.POST):
		year = request.POST.get('year', str(date_today.year))
		month = request.POST.get('month', str(date_today.month))
		day = request.POST.get('day', str(date_today.day))
		hour = request.POST.get('hour', str(date_today.hour))
	year = int(year)
	month = int(month)
	day = int(day)
	hour = int(hour)

	start_date = datetime.datetime(year, month, day, hour, 0, 0)

	traff=[]

	users = Users.objects.order_by('id')

	all_download = 0
	all_upload = 0
	all_cost_download = 0
	all_cost_upload = 0

	for user in users:
		#get download, upload and cost for it
		row = get_aggregate_stats(user.id, start_date, 'hour')[0]
		if row[0] is None:
			download = 0
		else:
			download = row[0]
		all_download = all_download + download

		if row[2] is None:
			upload = 0
		else:
			upload = row[2]
		all_upload = all_upload + upload

		if row[1] is None:
			cost_download = 0
		else:
			cost_download = row[1]
		all_cost_download = all_cost_download + cost_download
	
		if row[3] is None:
			cost_upload = 0
		else:
			cost_upload = row[3]
		all_cost_upload = all_cost_upload + cost_upload

		traff.append(UTraffic(user, download, upload, cost_download, cost_upload))

	traff.sort(key=operator.attrgetter('upload'),reverse=True)
	traff.sort(key=operator.attrgetter('download'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_upload'),reverse=True)
	traff.sort(key=operator.attrgetter('cost_download'),reverse=True)


	all_download = bytes2Xbytes(all_download)
	all_upload = bytes2Xbytes(all_upload)
	all_cost_download = "%1.2f" % (all_cost_download)
	all_cost_upload = "%1.2f" % (all_cost_upload)

	for tr in traff:
		tr.cost_download = "%1.2f" % (tr.cost_download)
		tr.cost_upload = "%1.2f" % (tr.cost_upload)
		tr.download = bytes2Xbytes(tr.download)
		tr.upload = bytes2Xbytes(tr.upload)

	#traff by filter
	rows = get_aggregate_stats_by_filter(0, start_date, "hour")
	tbfilter=get_traff_by_filter(rows)

	return render_to_response('traff/users_stats_hour.html',
			{'traff': traff,
			'month': month,
			'months': range(1,13),
			'year': year,
			'years': range(date_today.year-3,date_today.year+1),
			'day': day,
			'days': range(1,monthrange(year,month)[1]+1),
			'hour': hour,
			'hours': range(1,24),
			'all_download': all_download,
			'all_upload': all_upload,
			'all_cost_download': all_cost_download,
			'all_cost_upload': all_cost_upload,
			'traff_by_filter': tbfilter,
			'WITHOUT_COST': settings.WITHOUT_COST
			},
			context_instance=RequestContext(request)
			)


def bytes2Xbytes(bytes):
	if (bytes >= 1073741824):
		rez = "%1.3f Gb" % (bytes/1073741824)
	else:
		if (bytes >= 1048576):
			rez = "%1.2f Mb" % (bytes/1048576)
		else:
			if (bytes >= 1024):
				rez = "%1.1f Kb" % (bytes/1024)
			else:
				rez = "%s bytes" % (bytes)
	return rez

def get_traff_by_filter(rows):	
	tbfilter = []
	for row in rows:
		filter_title = ServiceTraffFilters.objects.get(pk=row[0]).title;
		bytes_download = bytes2Xbytes(row[1])
		bytes_upload = bytes2Xbytes(row[2])
		cost_download = "%1.2f" % (float(row[3]))
		cost_upload = "%1.2f" % (float(row[4]))
		tbfilter.append(traff_by_filter(filter_title, bytes_download, bytes_upload, cost_download, cost_upload))
	return tbfilter

def get_aggregate_stats(user_id, start_date, interval):
	cursor = connection.cursor()
	cursor.execute("SELECT  sum(bytes*in_traff) as bytes_in, sum(cost*in_traff) as cost_in, sum(bytes*(not in_traff)) as bytes_out, sum(cost*(not in_traff)) as cost_out FROM service_traff_stats_%s WHERE storetime='%s' AND userid=%s" % 
					(interval, start_date, user_id)
					)
	return cursor.fetchall()

def get_stats(user_id, start_date, interval, group_by, limit, fields=None):
	cursor = connection.cursor()

	if (limit == 0):
		limit_sql = ""
	else:
		limit_sql = "LIMIT %d" % limit

	if (user_id == 0):
		where_sql = ""
	else:
		where_sql = "AND userid=%d" % int(user_id)

	if (fields is None):
		if (group_by == 'addr'):
			addition_select_field = "INET_NTOA(addr)"
		else:
			addition_select_field = group_by
		fields_sql = "%s,sum(bytes*in_traff), sum(bytes*(not in_traff)), sum(cost*in_traff), sum(cost*(not in_traff))" % addition_select_field
	else:
		fields_sql = fields

	cursor.execute("SELECT %s FROM service_traff_stats WHERE storetime BETWEEN '%s' AND DATE_ADD('%s', INTERVAL 1 %s) %s GROUP BY %s ORDER BY sum(bytes) DESC %s" % 
					(fields_sql, start_date, start_date, interval, where_sql, group_by, limit_sql)
					)
	return cursor.fetchall()

def get_aggregate_stats_by_filter(user_id, start_date, interval):
	if (user_id == 0):
		where_sql = ""
	else:
		where_sql = "AND userid=%d" % int(user_id)
	cursor = connection.cursor()
	cursor.execute("SELECT  filterid, sum(bytes*in_traff), sum(bytes*(not in_traff)), sum(cost*in_traff), sum(cost*(not in_traff)) FROM service_traff_stats_%s WHERE storetime='%s' %s GROUP BY filterid ORDER BY sum(bytes) DESC" %
				(interval, start_date, where_sql)
				)
	return cursor.fetchall()
